/*
 This file is part of the digitalSTROM ds485-core library.

 Copyright (C) 2010 digitalSTROM.org, Schlieren, Switzerland

 digitalSTROM ds485-core library is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation version 2
 of the License.

 digitalSTROM ds485-core library is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with digitalSTROM ds485-core library; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

/**
 * @file
 * Provides the system message logger functionality
 */
#include    "logger.h"
#include    <stdarg.h>

/*
 * called by the dS485 protocol stack to send warning/debug/error
 * messages to upper system layers
 */
int ds485_system_msg(Ds485LibInst_t *dev, unsigned char level, const char *msg, ...)
{
    char buffer[512];
    va_list args;

    pthread_mutex_lock(&dev->logMutex);

    va_start(args, msg);

    if (dev->callback_msg_handler != NULL) {
        dev->callback_msg_handler(dev, level, msg, args);
    }
    else {
#ifdef DEBUG
        vsnprintf(buffer, 512, msg, args);
        printf("%s", buffer);
#endif
    }
    va_end(args);

    pthread_mutex_unlock(&dev->logMutex);
    return 0;
}
